package it.mokabyte.cache;

import it.mokabyte.cache.exception.NotFoundObjectException;
import it.mokabyte.cache.key.KeyObjectId;
import it.mokabyte.cache.locator.CacheServiceLocator;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;

/**
 *  Implementazione di una cache distribuita appoggiandosi al JNDI server
 *
 *@author     lucadozio
 *@created    19 maggio 2003
 */
public class JNDICache implements ICache {

    private Context ctx = null;


    /**
     *  Constructor for the JNDICache object
     */
    public JNDICache() {
        if (ctx == null) {
            try {
                ctx = CacheServiceLocator.getContext();
            } catch (NamingException ne) {
                ne.printStackTrace(System.err);
                ctx = null;
            }
        }
    }


    /**
     *  Il metodo getObjectInCache carica l'oggeto in cache cercando prima in
     *  locale e poi in distribuito
     *
     *@param  key                          chiave per il recupero dell'oggetto
     *@return                              Object in cache restituito
     *@exception  NotFoundObjectException  Description of the Exception
     *@throws  NotFoundObject              eccezzione generata nel caso non si
     *      torvi l'oggetto in cache
     */
    public Object get(KeyObjectId key) throws NotFoundObjectException {
        Object retObj = null;
        if (ctx != null) {
            retObj = getRemoteCacheObject(key.getId());
            System.err.println("tornato..." + retObj);
            if (null == retObj) {
                throw new NotFoundObjectException("L'oggetto [" + key.getId() + "] di classe [" + key.getInfoClassKey() + "] non  in cache");
            }
        }
        return retObj;
    }


    /**
     *  Il metodo addObject, aggiunge alla cache locale e remota l'oggetto
     *  recuperato dal flusso di esecuzione
     *
     *@param  key          la chiave per la registrazione dell'oggetto nella
     *      cache
     *@param  objToCache  Description of the Parameter
     */
    public void put(KeyObjectId key, Object objToCache) {
        if (ctx != null) {
            this.putRemoteObject(key.getId(), objToCache);
        }
    }


    /**
     *  rimozione dell'oggetto dalla cache
     *
     *@param  key                          chiave per la rimozione dell'oggetto
     *      dalla cache
     *@exception  NotFoundObjectException  l'oggetto non  nella cache
     */
    public void remove(KeyObjectId key) throws NotFoundObjectException {
        if (ctx != null) {
            removeRemote(key.getId());
        } else {
            throw new NotFoundObjectException("L'oggetto [" + key.getId() + "] di classe [" + key.getInfoClassKey() + "] non  in cache");
        }
    }


    /**
     *  metodo che effettua l'unbind dell'oggetto remoto
     *
     *@param  key  jndi name dell'oggetto in JNDI Server
     */
    private void removeRemote(String key) {
        //unbind
        System.out.println("rimozione remota..." + key);
        try {
            ctx.unbind(key);
        } catch (NameNotFoundException nnfe) {
            nnfe.printStackTrace(System.err);
            //throw new NotFoundObjectException("L'oggetto [" + key + "] ha provocato una Exc:" + nnfe.getMessage());
        } catch (NamingException ne) {
            ne.printStackTrace(System.err);
            //throw new NotFoundObjectException("L'oggetto [" + key + "] ha provocato una Exc:" + ne.getMessage());
        }
    }


    /**
     *  metodo che realizza il bind al JNDI Server dell'oggetto remoto.
     *
     *@param  key         jndi name di bind
     *@param  objToCache  oggetto da registrare in cache
     */
    private void putRemoteObject(String key, Object objToCache) {
        //aggiungo in distribuito
        try {
            try {
                ctx.bind(key, objToCache);
                System.err.println("bind --1--fatto!");
            } catch (javax.naming.NameAlreadyBoundException nabe) {
                nabe.printStackTrace(System.err);
                System.err.println("unbind...." + key);
                ctx.unbind(key);
                System.err.println("bind di nuovo ..." + key);
                ctx.bind(key, objToCache);
            }
        } catch (NamingException ne) {
            ne.printStackTrace(System.err);
        }
    }


    /**
     *  recupero l'oggetto dal JNDI server
     *
     *@param  key  jndi name per l'oggetto registrato sul JNDI Server
     *@return      oggetto remoto
     */
    private Object getRemoteCacheObject(String key) {
        Object retObj = null;
        try {
            retObj = ctx.lookup(key);
            if (retObj != null) {
                System.err.println("VERIFICA OBJ: " + retObj.getClass().getName());
            } else {
                System.err.println("VERIFICA OBJ: is NULL! Object Not Found");
            }
        } catch (NamingException ne) {
            ne.printStackTrace(System.err);
        }
        return retObj;
    }


}
