package it.mokabyte.cache.manager;

import it.mokabyte.cache.LocalCache;
import it.mokabyte.cache.JNDICache;
import it.mokabyte.cache.FactoryCache;
import it.mokabyte.cache.exception.NotFoundObjectException;
import it.mokabyte.cache.key.KeyCreator;
import it.mokabyte.cache.key.KeyObjectId;
import it.mokabyte.cache.policy.PolicyEventListener;
import it.mokabyte.cache.locator.CacheServiceLocator;

import javax.jms.Message;

import javax.jms.JMSException;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 *  Classe che realizza l'interfaccia per l'accesso al mondo Cache nonch i vari
 *  algoritmi di gestione delle policy del caching
 *
 *@author     lucadozio
 *@created    9 maggio 2003
 */
public class CacheManager implements PolicyEventListener {

    private static boolean enable = true;
    private TopicConnection topicConnection = null;
    private LocalCache lch;
    private JNDICache dch;


    /**
     *  Constructor for CacheManager.
     */
    public CacheManager() {
        super();
        initSubscriber();
        lch = (LocalCache) FactoryCache.getCache(FactoryCache.LOCAL);
        dch = (JNDICache) FactoryCache.getCache(FactoryCache.JNDI);
    }


    /**
     *  inizializzazione del topic subscriber
     */
    private void initSubscriber() {

        if (topicConnection == null) {
            try {
                Context jndiContext = CacheServiceLocator.getContext();
                System.out.println("JNDI Context[" + jndiContext + "]...");
                String topicConnectionFactoryName = "MokaTopicConnectionFactory";
                String topicName = "Luke";
                // Look up connection ConnectionFactory e detination Topic
                System.out.println("Looking up connection factory [" + topicConnectionFactoryName + "]...");
                TopicConnectionFactory topicConnectionFactory = (TopicConnectionFactory) jndiContext.lookup(topicConnectionFactoryName);
                System.out.println("Looking up topic [" + topicName + "]...");
                Topic topic = (Topic) jndiContext.lookup("topic/" + topicName);
                System.out.println("topicConnectionFactory [" + topicConnectionFactory + "] - topic [" + topic + "]");

                topicConnection = topicConnectionFactory.createTopicConnection();
                System.out.println("topicConnection [" + topicConnection + "]");
                TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
                System.out.println("topicSession [" + topicSession + "]");
                // Parte di Subscriber
                TopicSubscriber topicSubscriber = topicSession.createSubscriber(topic, null, true);
                System.out.println("topicSubscriber [" + topicSubscriber + "]");

                System.out.println("Creazione e installazione del listener...");

                topicSubscriber.setMessageListener(this);
                topicConnection.start();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            System.out.println("TOPIC CONNENCTION ALREADY CREATE!");
        }
    }


    /**
     *  il metodo si occupa di recuperare l'oggetto in cache, secondo la chaive
     *  di ricerca.
     *
     *@param  requestToPerform              l'oggetto da cui viene generata la
     *      chiave per la ricerca dell'oggetto in cache
     *@return                              oggetto che  stato recuperato dalla
     *      cache
     *@exception  NotFoundObjectException  l'oggetto in questione NON  presente
     *      nella cache.
     */
    public Object fetchObject(Object requestToPerform) throws NotFoundObjectException {
        if (!enable) {
            throw new NotFoundObjectException("Cache is <<DISABLED>>");
        }
        //richiedo al key creator
        KeyObjectId key = KeyCreator.getInstance().getKey(requestToPerform);
        //richiedo alla cache
        try {
            //local
            return lch.get(key);
        } catch (NotFoundObjectException nfo) {
            //remota
            Object cached = dch.get(key);
            //avendo trovato l'oggetto in remoto lo duplico in locale
            lch.put(key, cached);
            return cached;
        }

    }


    /**
     *  il metodo inserisce l'oggetto nella cache
     *
     *@param  keyToRecordObject  oggetto utilizzato pe rla generazione della
     *      chiave di memorizzazione
     *@param  objToCache         oggetto da salvare nella cache
     */
    public void putObject(Object keyToRecordObject, Object objToCache) {
        if (!enable) {
            return;
        }
        KeyObjectId key = KeyCreator.getInstance().getKey(keyToRecordObject);
        dch.put(key, objToCache);
        lch.put(key, objToCache);
    }


    /**
     *  rimozione dell'oggetto dalla cache
     *
     *@param  keyToSearchInCache           oggetto chiave per la ricerca
     *      dell'oggetto nella cache
     *@exception  NotFoundObjectException  l'oggetto cercato non  presente
     *      nella cache
     */
    public void removeObject(Object keyToSearchInCache) throws NotFoundObjectException {
        if (!enable) {
            return;
        }
        KeyObjectId key = KeyCreator.getInstance().getKey(keyToSearchInCache);
        dch.remove(key);
        lch.remove(key);

    }


    /**
     *  metodo che rimuove tutti gli oggetti in cache
     */
    public void removeAll() {
        if (!enable) {
            return;
        }
        System.out.println("....remove all....");
        java.util.Set key = lch.getKeySet();
        java.util.Iterator ikey = key.iterator();
        KeyObjectId keyId;
        while (ikey.hasNext()) {
            keyId = (KeyObjectId) ikey.next();
            try {
                dch.remove(keyId);
            } catch (Exception e) {
                //azzittita
            }
        }
        lch.removeAll();
    }


    /**
     *  gestione delle policy di indirizzamento della cache. <BR>
     *  Situazioni gestite ad oggi:
     *  <LI> DISABLE --> disabilita la cache
     *  <LI> ENABLE --> abilita la cache
     *  <LI> REFRESH --> rinfresca la cache
     *
     *@param  policyMsgEvent  evento di politica della gestione cache
     */
    public void onMessage(Message policyMsgEvent) {
        System.out.println("Soggetta ad implementazione.......");
        try {
            if (policyMsgEvent instanceof TextMessage) {
                TextMessage txtMsg = (TextMessage) policyMsgEvent;
                System.out.println("Ricevuto -->[" + txtMsg.getText() + "]");
                if (txtMsg.getText().trim().equalsIgnoreCase("DISABLE")) {
                    enable = false;
                }
                if (txtMsg.getText().trim().equalsIgnoreCase("ENABLE")) {
                    enable = true;
                }
                if (txtMsg.getText().trim().equalsIgnoreCase("REFRESH")) {
                    removeAll();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            //nel caso di potenziali problemi rimuovo tutti i componenti dalla cache
            removeAll();
            //e la disabilito...
            enable = false;
        }
    }

}
